/**
 * Interface that represents lists of mergeable elements (supporting merge).
 * 
 * @param <E> type of the list
 *
 * @author Antonia Lopes 
 * @version 1.0 04/Feb/2013
 */
public interface ListWithMerge<E extends Mergeable<E>> extends Iterable<E> {
	
    /**
     * Add element to the beginning of the list
     * 
     * @param e - Element to be added
     */
    void addFirst(E e);

    /**
     * Remove the element at index i
     * 
     * @param i - Index of element to be removed
     * @requires i >= 0 && i < size()
     */
    void remove(int i);

    /**
     * Get the element in the index i
     * 
     * @param i - Index of element to be obtained
     * @return - element at the indicated index
     */
    E get(int i);

    /**
     * Set the index i with element e
     * 
     * @param i - Index to be set
     * @param e - Element to be set
     * @requires i >= 0 && i < size()
     */
    void set(int i, E e);

    /**
     * Returns the number of elements in the list
     * 
     * @return The number of elements in the list
     */
    int size();

    /**
     * Is the list empty?
     * 
     * @return true if the list is empty, false otherwise
     */
    boolean isEmpty();

    /**
     * Does the list contain the given element?
     * @param e -  Element to search
     * @return true if the given element is in the list, false otherwise
     */
    boolean contains(E e);

    /**
     * Merges all elements in the range [i,j]
     * 
     * @param i - initial index
     * @param j - final index
     * @requires 0 < i && i < j && j < size()
     */
    void mergeInRange(int i, int j);
}
