import java.util.Random;

/**
 * A class to randomly test a List with Merge
 * 
 * @author Gloss
 * @version 1.1 17/Jul/2013 as
 */
public class MergeListRandomTest {

	/**
	 * Random object to generate numbers
	 */
	private final Random rand;

	/**
	 * The list under testing
	 */
	private MyListWMerge<MyString> list;

	/**
	 * How many tests to perform
	 */
	private final int howMany;

	/**
	 * Maximum number of operations
	 */
	private static final int MAX_OPERATIONS = 8;

	/**
	 * Maximum number of values on the list
	 */
	private final static int MAX_NUMBER_OF_VALUES = 8;

	/**
	 * Constructor
	 * @param list - the list to be tested
	 * @param howMany - how many operation tests to perform
	 */
	public MergeListRandomTest(MyListWMerge<MyString> list, int howMany) {
		this.list = list;
		this.howMany = howMany;
		this.rand = new Random();
	}

	/**
	 * A MyString to place in the list
	 * @return a random MyString
	 */
	private MyString newMyString() {
		Integer i = rand.nextInt(MAX_NUMBER_OF_VALUES);
		String s = i.toString();
		return new MyString(s);
	}

	/**
	 * Call howMany operations on the list. 
	 * Select randomly each operation to call. 
	 * Checks the preconditions of each call.
	 */
	public void run() {
		int index = -1;
		MyString string = null;
		
		for (int i = 0; i < howMany; i++) {
			int size = list.size();
			
			switch (rand.nextInt(MAX_OPERATIONS)) {
			
			case 0:
				System.out.println("make");
				list = new MyListWMerge<MyString>();
				break;
				
			case 1:
				string = newMyString();
				System.out.println("add: " + string);
				list.addFirst(string);
				break;
				
			case 2:
				if (size > 0) {
					index = rand.nextInt(size);
					System.out.println("get " + index + ": " + list.get(index));
				}
				break;
				
			case 3:
				if (size > 0) {
					index = rand.nextInt(size);
					System.out.println("remove " + index);
					list.remove(index);
				}
				break;
				
			case 4:
				System.out.println("isEmpty: " + list.isEmpty());
				break;
				
			case 5:
				System.out.println("size: " + list.size());
				break;
				
			case 6:
				if (size > 0) {
					index = rand.nextInt(size);
					string = newMyString();
					System.out.println("set " + string + " at index " + index);
					list.set(index, string);
				}
				break;
				
			case 7:
				if (size > 0) {
					int k = rand.nextInt(size);
					if (k < size-1){
						int j = rand.nextInt(size-k-1)+1;
						System.out.println("mergeinrange " + k + " "+ (k+j));
						list.mergeInRange(k,k+j);
					}
				}
				break;
			}
			
			System.out.println(list);
		}
		System.out.println("done!");
	}
}