import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Implementation of a List with Merge
 * 
 * @author Gloss
 * @version 1.1 17/Jul/2013 as
 * @param <E> Type of the list
 */
public class MyListWMerge<E extends Mergeable<E>> implements ListWithMerge<E>, Cloneable {
	
	/**
	 * The list of elements
	 */
	private List<E> list;

	/**
	 * Constructor
	 */
	public MyListWMerge() {
		list = new ArrayList<E>();
	}

	/**
	 * Add element in the beginning of the list
	 * @param e - element to be added
	 * @requires e != null
	 */
	public void addFirst(E e) {
		list.add(0, e);
	}

	 /**
     * Does the list contain the given element?
     * @param e -  Element to search
     * @return true if the given element is in the list, false otherwise
     */
	public boolean contains(E e) {
		return list.contains(e);
	}
	
	 /**
     * Merges all elements in the range [i,j]
     * 
     * @param i - initial index
     * @param j - final index
     * @requires 0 < i && i < j && j < size()
     */
	public void mergeInRange(int i, int j) {
		if (i < j) {
			E element = list.get(i);
			for (int k = i + 1; k <= j; k++)
				element.merge(list.get(k));
			list.set(i, element);
			int k = i + 1;
			while (k <= j){
				list.remove(k);
				j--;
			}
		}
	}
	
    /**
     * Returns the number of elements in the list
     * 
     * @return The number of elements in the list
     */
	public int size() {
		return list.size();
	}

	/**
	 * Returns the iterator for the list
	 * 
	 * @return iterator for the list
	 */
	public Iterator<E> iterator() {
		return list.iterator();
	}

	 /**
     * Get the element in the index i
     * 
     * @param i - Index of element to be obtained
     * @return element at the indicated index
     */
	public E get(int i) {
		return list.get(i);
	}

    /**
     * Remove the element at index i
     * 
     * @param i - Index of element to be removed
     * @requires i >= 0 && i < size()
     */
	public void remove(int i) {
		list.remove(i);
	}
	
    /**
     * Set the index i with element e
     * 
     * @param i - Index to be set
     * @param e - Element to be set
     * @requires i >= 0 && i < size() && e != null
     */
	public void set(int i, E e) {
		list.set(i, e);
	}
	
    /**
     * Is the list empty?
     * 
     * @return true if the list is empty, false otherwise
     */
	public boolean isEmpty() {
		return size() == 0;
	}

	/**
	 * Returns a copy of this list
	 * 
	 * @return clone of the list
	 */
	@SuppressWarnings("unchecked")
	@Override
	public MyListWMerge<E> clone() {
		MyListWMerge<E> clone = null;
		try {
			clone = (MyListWMerge<E>) super.clone();
			clone.list = (List<E>) ((ArrayList<E>) this.list).clone(); 
			for (int i = 0; i < list.size(); i++) {
				E elem = clone.list.get(i);
				clone.list.set(i, elem.clone());
			}
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return clone;
	}

	/**
	 * Is this list equal to the object?
	 * 
	 * @param obj - the object to compare
	 * @return true if the objects are equal
	*/
	@SuppressWarnings("unchecked")
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof MyListWMerge) {
			MyListWMerge<E> other = (MyListWMerge<E>) obj;
			return other == this || this.list.equals(other.list);
		}
		return false;
	} 

	/**
	 * Returns the hash code for the list
	 *
	 * @return hash code for this list
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((list == null) ? 0 : list.hashCode());
		return result;
	}

	/**
	 * Textual representation of the list
	 * 
	 * @return String with the representation of the list
	 */
	@Override
	public String toString() {
		StringBuilder result = new StringBuilder();
		result.append('[');
		for (E elem : list)
			result.append(elem + ", ");
		if (result.length() >= 2)
			result.delete(result.length()-2, result.length());
		result.append(']');
		return result.toString();
	}
}
