/**
 * The class that implements a mutable version of strings that are mergeable.
 * 
 * @author Gloss
 * @version 1.1 - 17/Jul/2013 as
 **/

public class MyString implements Mergeable<MyString>, Cloneable{

	/**
	 * The String represented by MyString
	 */
	private String myString;
	
	/**
	 * Constructor
	 * @param string - String to be represented
	 */
	public MyString(String string){
		myString = string;
	}
	
	/**
	 * Merges this MyString with another
	 * @param other - the MyString to merge with this
	 */
	@Override
	public void merge(MyString other) {
		myString = this.myString + other.myString;
	}

	/**
	 * MyString textual representation
	 * 
	 * @return the String representation.
	 */
	@Override
	public String toString() {
		return myString;
	}
	
	/**
	 * The hashCode for MyString.
	 * 
	 * @return the MyString hashCode.
	 **/
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((myString == null) ? 0 : myString.hashCode());
		return result;
	}
	
	/**
	 * Returns a copy of this MyString
	 * @return clone of this MyString
	 */
	public MyString clone() {
		MyString clone = null;
		try {
			clone = (MyString) super.clone();
			clone.myString = new String(myString);
		} catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return clone;
	}
	
	/**
	 * Compare this MyString with a given object.
	 * 
	 *  @param obj - the given object.
	 *  @return true if the given object is equal to this MyString, false otherwise.
	 **/
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		MyString other = (MyString) obj;
		if (myString == null) {
			if (other.myString != null)
				return false;
		} else if (!myString.equals(other.myString))
			return false;
		return true;
	}
}